@@ -1,6 +1,6 @@
 # coding: utf-8
 
-from vxci.get_addons import get_module_list, get_modules2test
+from vxci.get_addons import get_module_list, get_modules2test, get_tags, join_prefix_suffix
 from vxbase import vtypes, net, fromenv
 from vxci.tools_test import install_chrome_commands
 import distutils.spawn
@@ -85,14 +85,18 @@ def run_build_image(config):
     image_sha = images[0].get('Id')
     short_id = image_sha.split(':')[1][:10]
     tag = short_id
+    git_sha = config["git_commit_sha"][:7]  # 7 is the default size "git rev-parse --short HEAD"
     if config.get('docker_image_repo'):
         tag = "%s-%s-%s" % (config['main_app'], config['version'], short_id)
+        git_tag = "%s-%s-%s" % (config['main_app'], config['version'], git_sha)
         if is_dev_repo(config):
             tag = "%s-%s-dev%s" % (config['main_app'], config['version'], short_id)
-    config.update({'image_tag': tag})
+    config.update({'image_tag': tag, 'image_git_tag': git_tag})
 
 
 def clear_images(config):
+    if config.get("vxci_no_clear_images"):
+        return
     images = [config['instance_image'], config['postgres_image'], ]
     for image in images:
         try:
@@ -106,6 +110,8 @@ def clear_images(config):
 
 def clean_containers(config):
     """ Cleans any running container related to the same build to avoid any conflicts """
+    if config.get("vxci_no_clear_containers"):
+        return
     containers = _cli.containers(all=True, filters={'name': config['base_name']})
     for container in containers:
         try:
@@ -377,6 +383,14 @@ def save_imagename(config):
 
 
 def start_instance(config):
+    coverage_info = {
+        "COVERAGE_RCFILE": "%(coverage_reports)s/.coveragerc" % config,
+        "COVERAGE_FILE": "%(coverage_reports)s/.coverage" % config,
+        "COVERAGE_HOME": "/home/odoo/instance/extra_addons/%(ci_project_name)s" % config,
+        "COVERAGE_CONTEXT": "job_%(ci_job_name)s" % config,
+        "EXCLUDE_COVERAGE": config.get("exclude_coverage"),
+        "COVERAGE_PRECISION": config.get("coverage_precision"),
+    }
     env = {
         "DB_USER": "odoo",
         "DB_PASSWORD": "odoo",
@@ -384,8 +398,9 @@ def start_instance(config):
         "ODOO_CONFIG_FILE": "/home/odoo/.openerp_serverrc",
         # Enable this to debug coverage
         # "COVERAGE_DEBUG": "config,dataio,process",
-        "COVERAGE_RCFILE": "/home/odoo/.coveragerc"
     }
+    env.update(coverage_info)
+    config.update({key.lower(): value for key, value in coverage_info.items()})
     for env_var in ['COUNTRY', 'LANGUAGE']:
         env.update({env_var: environ.get(env_var, "")})
     links = {
@@ -433,7 +448,7 @@ def exec_cmd(container, cmd, user=None, stream=False, workdir=None, tty=False):
         _logger.info("--->> Start of the stream <<---")
         for line in res:
             line = vtypes.decode(line)
-            print(line.strip('\n'))
+            print(line.strip('\n'), flush=True)
             lines.append(line)
         _logger.info("--->> End of the stream <<---")
         return lines
@@ -520,16 +535,16 @@ def install_module(config):
 
 def test_module(config):
     modules2test = get_modules2test('.', config.get('include'), config.get('exclude'))
+    odoo_test_tags = set(i.strip() for i in (config.get('odoo_test_tags') or '').strip().split(',')) - {''}
+    tags = get_tags(modules2test, odoo_test_tags)
     _logger.debug("module: %s", modules2test)
     extra = ''
     if config.get('language', False):
         extra += ' --load-language={lang}'.format(lang=config.get('language'))
-    tags = ",/".join(modules2test)
-    extra_tags = config.get('odoo_test_tags')
-    if extra_tags:
-        tags += ',%s' % extra_tags
-    extra += ' --test-tags=/{tags}'.format(tags=tags)
-    _logger.debug("test tags: %s", tags)
+
+    extra += ' --test-tags={tags}'.format(tags=tags)
+    _logger.debug("extra: %s", extra)
+
     test_command = (
         "unbuffer coverage run /home/odoo/instance/odoo/odoo-bin -d test -i {mod} --test-enable --workers 0 "
         "{extra} --stop-after-init".format(mod=','.join(modules2test), extra=extra)
@@ -542,7 +557,7 @@ def test_module(config):
 
     _logger.info('\nRunning tests for %s', modules2test)
     _logger.info('Command : %s', test_command)
-    test_res = exec_cmd(config['container_name'], test_command, 'odoo', stream=True, workdir=config.get('coverage_workdir'))
+    test_res = exec_cmd(config['container_name'], test_command, 'odoo', stream=True, workdir=config.get('coverage_home'))
 
     test_log = resume_log(test_res, config['allow_deprecated'])
     show_log(test_log[1], 'Installation with demo and tests', config)
@@ -560,7 +575,7 @@ def test_module(config):
     chdir(config.get('container_vol'))
     for cr in cover_reports:
         _logger.info('Running command: %s', cr)
-        cmd_res = exec_cmd(config['container_name'], cr, 'odoo', stream=False, workdir=config.get('coverage_workdir'))
+        cmd_res = exec_cmd(config['container_name'], cr, 'odoo', stream=False, workdir=config.get('coverage_home'))
         _logger.info('Command: "%s" exited with:\n%s', cr, cmd_res)
 
     reports_getters =[
@@ -590,16 +605,17 @@ def prepare_coverage(config):
         makedirs(config.get('container_vol'))
     fcoveragerc = path.join(config.get('container_vol'), '.coveragerc')
     with open(fcoveragerc, "w") as f_coveragerc:
-        f_coveragerc.write(get_coverage_content(config.get('ci_project_name')))
+        f_coveragerc.write(get_coverage_content(config))
+
+    _logger.debug('Create workdir %s', exec_cmd(config['container_name'], 'mkdir -p {coverage_reports}'.format(**config), user='odoo'))
 
     cp_cover_cmd = 'docker cp {coverrc} {container}:{folder}'.format(
         coverrc=fcoveragerc,
         container=config.get('container_name'),
-        folder=config.get('coverage_workdir'))
+        folder=config.get('coverage_rcfile'))
     subprocess.call(shlex.split(cp_cover_cmd))
 
-    _logger.debug('Changed permissions for coverage %s', exec_cmd(config['container_name'], 'chown odoo:odoo {coveragerc}'.format(coveragerc=path.join(config.get('coverage_workdir'), '.coveragerc'))))
-    _logger.debug('Create workdir %s', exec_cmd(config['container_name'], 'mkdir {report}'.format(report=config.get('coverage_reports')), user='odoo'))
+    _logger.debug('Changed permissions for coverage %s', exec_cmd(config['container_name'], 'chown odoo:odoo {coverage_rcfile}'.format(**config)))
 
     _logger.debug('Installing dependencies to run tests')
     exec_cmd(config.get('container_name'), 'touch /home/odoo/full_test-requirements.txt')
@@ -620,46 +636,11 @@ def save_logs(config, logs):
             fodoo_log.write(line)
 
 
-def get_coverage_content(config_main_app):
-    coveragerc_content = """
-# Config file .coveragerc
-
-[report]
-include =
-    /home/odoo/instance/extra_addons/{main_app}/*
-
-omit =
-    */scenario/*
-    */scenarios/*
-    */test/*
-    */tests/*
-    *_example/*
-    *__init__.py
-    *__openerp__.py
-    *__manifest__.py
-    {exclude_var}
-
-# Regexes for lines to exclude from consideration
-exclude_lines =
-    # Have to re-enable the standard pragma
-    pragma: no cover
-    # Don't complain about null context checking
-    if context is None:
-    # Don't check obviously not implemented
-    raise NotImplementedError
-    # We don't really care what happens if fail
-    except ImportError:
-
-fail_under = {min_var}
-precision = {precision_var}
-
-    """.format(
-        main_app=config_main_app,
-        workdir='/tmp',
-        exclude_var='${EXCLUDE_COVERAGE}',
-        min_var='${COVERAGE_MIN-0}',
-        precision_var='${COVERAGE_PRECISION-0}',
-    )
+def get_coverage_content(config):
+    modules2test = get_modules2test('.', config.get('include'), config.get('exclude'))
+    include_coverage = "%s/*" % join_prefix_suffix(modules2test, "./", "/*,")
+    with open(path.join(path.abspath(path.dirname(__file__)), "templates", "coveragerc"), "r") as f_coveragerc:
+        coveragerc_content = f_coveragerc.read().format(include_coverage=include_coverage)
     return coveragerc_content
 
 def show_log(log, title, config):
